@@ -10,12 +10,10 @@
 # FOR A PARTICULAR PURPOSE
 # 
 ##############################################################################
+""" Basic Site content type registry
 
+$Id: ContentTypeRegistry.py,v 1.9.4.2 2002/08/04 02:42:22 efge Exp $
 """
-Basic Site content type registry
-$Id: ContentTypeRegistry.py,v 1.7 2001/11/29 21:47:35 tseaver Exp $
-"""
-__version__='$Revision: 1.7 $'[11:-2]
 
 from OFS.SimpleItem import SimpleItem, Item
 from AccessControl import ClassSecurityInfo
@@ -23,7 +21,7 @@ from Globals import DTMLFile, InitializeClass, PersistentMapping
 from ZPublisher.mapply import mapply
 
 from CMFCorePermissions import ManagePortal
-from utils import _dtmldir
+from utils import _dtmldir, getToolByName
 
 import re, os, string, urllib
 
@@ -406,9 +404,12 @@ class ContentTypeRegistry( SimpleItem ):
         typeName = self.findTypeName( name, content_type, body )
         if typeName is None:
             typeName = '<unknown>'
+        else:
+            types_tool = getToolByName(self, 'portal_types')
+            typeName = types_tool.getTypeInfo(typeName).Title()
         REQUEST[ 'RESPONSE' ].redirect( self.absolute_url()
                                + '/manage_testRegistry'
-                               + '?testResults=Type:%s'
+                               + '?testResults=Type:+%s'
                                        % urllib.quote( typeName )
                                )
 
@@ -462,7 +463,7 @@ class ContentTypeRegistry( SimpleItem ):
         self.predicates[ predicate_id ] = ( klass( predicate_id ), None )
         self.predicate_ids = self.predicate_ids + ( predicate_id, )
 
-    security.declareProtected( ManagePortal, 'addPredicate' )
+    security.declareProtected( ManagePortal, 'updatePredicate' )
     def updatePredicate( self, predicate_id, predicate, typeObjectName ):
         """
             Update a predicate in this element.
