@@ -10,25 +10,24 @@
 # FOR A PARTICULAR PURPOSE
 # 
 ##############################################################################
+""" Type registration tool.
 
+$Id: TypesTool.py,v 1.37.4.7 2002/08/05 21:20:43 shane Exp $
 """
-    Type registration tool.
-    $Id: TypesTool.py,v 1.26.2.4 2002/01/09 22:28:12 tseaver Exp $
-"""
-__version__='$Revision: 1.26.2.4 $'[11:-2]
 
 import OFS
 from Globals import InitializeClass, DTMLFile
 from utils import UniqueObject, SimpleItemWithProperties, tuplize
 from utils import _dtmldir, _checkPermission, cookString, getToolByName
 import string
-from AccessControl import getSecurityManager, ClassSecurityInfo
-try:
-    from AccessControl import Unauthorized
-except:
-    Unauthorized = 'Unauthorized'
+from AccessControl import getSecurityManager, ClassSecurityInfo, Unauthorized
 from Acquisition import aq_base
 import Products, CMFCorePermissions
+from ActionProviderBase import ActionProviderBase
+from ActionInformation import ActionInformation
+from Expression import Expression
+from zLOG import LOG, WARNING, ERROR
+import sys
 
 from CMFCorePermissions import View, ManagePortal, AccessContentsInformation
 
@@ -59,15 +58,17 @@ class TypeInformation (SimpleItemWithProperties):
          'label':'Title'},
         {'id':'description', 'type': 'text', 'mode':'w',
          'label':'Description'},
-        {'id':'content_meta_type', 'type': 'string', 'mode':'w',
-         'label':'Meta type'},
         {'id':'content_icon', 'type': 'string', 'mode':'w',
          'label':'Icon'},
+        {'id':'content_meta_type', 'type': 'string', 'mode':'w',
+         'label':'Product meta type'},
         )
 
     _advanced_properties = (
         {'id':'immediate_view', 'type': 'string', 'mode':'w',
          'label':'Initial view name'},
+        {'id':'global_allow', 'type': 'boolean', 'mode':'w',
+         'label':'Implicitly addable?'},
         {'id':'filter_content_types', 'type': 'boolean', 'mode':'w',
          'label':'Filter content types?'},
         {'id':'allowed_content_types'
@@ -89,6 +90,7 @@ class TypeInformation (SimpleItemWithProperties):
     filter_content_types = 1
     allowed_content_types = ()
     allow_discussion = 0
+    global_allow = 1
     _actions = ()
 
     def __init__(self, id, **kw):
@@ -120,17 +122,21 @@ class TypeInformation (SimpleItemWithProperties):
     #
     security.declareProtected(View, 'Type')
     def Type(self):
+        """ Deprecated. Use Title(). """
+        LOG('CMFCore.TypesTool', WARNING,
+            'TypeInformation.Type() is deprecated, use Title().')
+        return self.Title()
+
+    security.declareProtected(View, 'Title')
+    def Title(self):
         """
             Return the "human readable" type name (note that it
-            may not map exactly to the 'meta_type', e.g., for
+            may not map exactly to the 'portal_type', e.g., for
             l10n/i18n or where a single content class is being
             used twice, under different names.
         """
-        if self.title:
-            return self.title
-        else:
-            return self.getId()
-    
+        return self.title or self.getId()
+
     security.declareProtected(View, 'Description')
     def Description(self):
         """
@@ -162,9 +168,20 @@ class TypeInformation (SimpleItemWithProperties):
             type object we are?
         """
         if not self.filter_content_types:
+            ti = self.getTypeInfo( contentType )
+            if ti is None or ti.globalAllow():
+                return 1
+
+        if contentType in self.allowed_content_types:
             return 1
-        return contentType in self.allowed_content_types
-    
+
+        # Backward compatibility for code that expected Type() to work.
+        for ti in self.listTypeInfo():
+            if ti.Title() == contentType:
+                return ti.getId() in self.allowed_content_types
+
+        return 0
+
     security.declarePublic('getId')
     def getId(self):
         return self.id
@@ -184,6 +201,13 @@ class TypeInformation (SimpleItemWithProperties):
         # Private because this returns the actual structure.
         return self._actions
 
+    security.declarePublic('globalAllow')
+    def globalAllow(self):
+        """
+        Should this type be implicitly addable anywhere?
+        """
+        return self.global_allow
+
     security.declarePublic('getActionById')
     def getActionById( self, id, default=_marker ):
         """
@@ -254,14 +278,19 @@ class TypeInformation (SimpleItemWithProperties):
         """
         if not name:
             raise ValueError('A name is required.')
-        self._actions = self._actions + (
-            {'id': str(id),
-             'name': str(name),
-             'action': str(action),
-             'permissions': (str(permission),),
-             'category': str(category),
-             'visible': int(visible),
-             },)
+
+        new_actions = self._cloneActions()
+
+        new_actions.append( { 'id'          : str(id)
+                            , 'name'        : str(name)
+                            , 'action'      : str(action)
+                            , 'permissions' : (str(permission),)
+                            , 'category'    : str(category)
+                            , 'visible'     : int(visible) 
+                            } )
+
+        self._actions = tuple( new_actions )
+
         if REQUEST is not None:
             return self.manage_editActionsForm(
                 REQUEST, manage_tabs_message='Added.')
@@ -276,19 +305,20 @@ class TypeInformation (SimpleItemWithProperties):
         actions = []
         for idx in range(len(self._actions)):
             s_idx = str(idx)
-            action = {
-                'id': str(properties.get('id_' + s_idx, '')),
-                'name': str(properties.get('name_' + s_idx, '')),
-                'action': str(properties.get('action_' + s_idx, '')),
-                'permissions':
-                (properties.get('permission_' + s_idx, ()),),
-                'category': str(properties.get('category_' + s_idx, '')),
-                'visible': not not properties.get('visible_' + s_idx, 0),
-                }
+            action = self._actions[idx].copy()
+            action.update( {
+                    'id': str(properties.get('id_' + s_idx, '')),
+                    'name': str(properties.get('name_' + s_idx, '')),
+                    'action': str(properties.get('action_' + s_idx, '')),
+                    'permissions':
+                    (properties.get('permission_' + s_idx, ()),),
+                    'category': str(properties.get('category_' + s_idx, '')),
+                    'visible': not not properties.get('visible_' + s_idx, 0),
+                    } )
             if not action['name']:
                 raise ValueError('A name is required.')
-            actions.append(action)
-        self._actions = tuple(actions)
+            actions.append( action )
+        self._actions = tuple( actions )
         if REQUEST is not None:
             return self.manage_editActionsForm(REQUEST, manage_tabs_message=
                                                'Actions changed.')
@@ -298,13 +328,17 @@ class TypeInformation (SimpleItemWithProperties):
         """
         Deletes actions.
         """
-        actions = list(self._actions)
         sels = list(map(int, selections))  # Convert to a list of integers.
         sels.sort()
         sels.reverse()
+
+        new_actions = self._cloneActions()
+
         for idx in sels:
-            del actions[idx]
-        self._actions = tuple(actions)
+            del new_actions[idx]
+
+        self._actions = tuple(new_actions)
+
         if REQUEST is not None:
             return self.manage_editActionsForm(
                 REQUEST, manage_tabs_message=(
@@ -315,19 +349,23 @@ class TypeInformation (SimpleItemWithProperties):
         """
         Moves the specified actions up one slot.
         """
-        actions = list(self._actions)
         sels = list(map(int, selections))  # Convert to a list of integers.
         sels.sort()
+
+        new_actions = self._cloneActions()
+
         for idx in sels:
             idx2 = idx - 1
             if idx2 < 0:
                 # Wrap to the bottom.
-                idx2 = len(actions) - 1
+                idx2 = len(new_actions) - 1
             # Swap.
-            a = actions[idx2]
-            actions[idx2] = actions[idx]
-            actions[idx] = a
-        self._actions = tuple(actions)
+            a = new_actions[idx2]
+            new_actions[idx2] = new_actions[idx]
+            new_actions[idx] = a
+
+        self._actions = tuple(new_actions)
+
         if REQUEST is not None:
             return self.manage_editActionsForm(
                 REQUEST, manage_tabs_message=(
@@ -338,25 +376,51 @@ class TypeInformation (SimpleItemWithProperties):
         """
         Moves the specified actions down one slot.
         """
-        actions = list(self._actions)
         sels = list(map(int, selections))  # Convert to a list of integers.
         sels.sort()
         sels.reverse()
+
+        new_actions = self._cloneActions()
+
         for idx in sels:
             idx2 = idx + 1
-            if idx2 >= len(actions):
+            if idx2 >= len(new_actions):
                 # Wrap to the top.
                 idx2 = 0
             # Swap.
-            a = actions[idx2]
-            actions[idx2] = actions[idx]
-            actions[idx] = a
-        self._actions = tuple(actions)
+            a = new_actions[idx2]
+            new_actions[idx2] = new_actions[idx]
+            new_actions[idx] = a
+
+        self._actions = tuple(new_actions)
+
         if REQUEST is not None:
             return self.manage_editActionsForm(
                 REQUEST, manage_tabs_message=(
                 'Moved down %d action(s).' % len(sels)))
 
+    security.declarePrivate( '_cloneActions' )
+    def _cloneActions( self ):
+        """
+            Return a "deep copy" of our list of actions.
+        """
+        return map( lambda x: x.copy(), list( self._actions ) )
+
+    security.declarePrivate('_finishConstruction')
+    def _finishConstruction(self, ob):
+        """
+            Finish the construction of a content object.
+            Set its portal_type, insert it into the workflows.
+        """
+        if hasattr(ob, '_setPortalTypeName'):
+            ob._setPortalTypeName(self.getId())
+            ob.reindexObject(idxs=['portal_type', 'Type'])
+
+        if hasattr(aq_base(ob), 'notifyWorkflowCreated'):
+            ob.notifyWorkflowCreated()
+
+        return ob
+
 InitializeClass( TypeInformation )
 
 
@@ -371,7 +435,7 @@ class FactoryTypeInformation (TypeInformation):
         {'id':'product', 'type': 'string', 'mode':'w',
          'label':'Product name'},
         {'id':'factory', 'type': 'string', 'mode':'w',
-         'label':'Factory method in product'},
+         'label':'Product factory method'},
         ) + TypeInformation._advanced_properties)
 
     product = ''
@@ -380,43 +444,63 @@ class FactoryTypeInformation (TypeInformation):
     #
     #   Agent methods
     #
-    def _getFactoryMethod(self, container, raise_exc=0):
+    def _getFactoryMethod(self, container):
         if not self.product or not self.factory:
-            return None
+            raise ValueError, ('Product factory for %s was undefined' %
+                               self.getId())
+        p = container.manage_addProduct[self.product]
+        m = getattr(p, self.factory, None)
+        if m is None:
+            raise ValueError, ('Product factory for %s was invalid' %
+                               self.getId())
+        if getSecurityManager().validate(p, p, self.factory, m):
+            return m
+        raise Unauthorized, ('Cannot create %s' % self.getId())
+
+    def _queryFactoryMethod(self, container, default=None):
+        if not self.product or not self.factory:
+            return default
         try:
             p = container.manage_addProduct[self.product]
             m = getattr(p, self.factory, None)
-            if m is not None:
+            if m is None:
+                return default
+            try:
+                # validate() can either raise Unauthorized or return 0 to
+                # mean unauthorized.
                 if getSecurityManager().validate(p, p, self.factory, m):
                     return m
-            return None
-        except: # only raise if allowed
-            if raise_exc:
-                raise
-            return None
+            except Unauthorized:
+                pass
+            return default
+        except:
+            LOG('Types Tool', ERROR, '_queryFactoryMethod raised an exception',
+                error=sys.exc_info())
+        return default
 
     security.declarePublic('isConstructionAllowed')
-    def isConstructionAllowed ( self, container, raise_exc=0):
+    def isConstructionAllowed ( self, container ):
         """
-        Does the current user have the permission required in
-        order to construct an instance?
+        a. Does the factory method exist?
+
+        b. Is the factory method usable?
+
+        c. Does the current user have the permission required in
+        order to invoke the factory method?
         """
-        m = self._getFactoryMethod(container, raise_exc)
+        m = self._queryFactoryMethod(container)
         return (m is not None)
 
     security.declarePublic('constructInstance')
     def constructInstance( self, container, id, *args, **kw ):
         """
         Build a "bare" instance of the appropriate type in
-        'container', using 'id' as its id.  Return the URL
-        of its "immediate" view (typically the metadata form).
+        'container', using 'id' as its id.  Return the object.
         """
         # Get the factory method, performing a security check
         # in the process.
-        m = self._getFactoryMethod(container, raise_exc=1)
 
-        if m is None:
-            raise Unauthorized, ('Cannot create %s' % self.getId())
+        m = self._getFactoryMethod(container)
 
         id = str(id)
 
@@ -429,12 +513,7 @@ class FactoryTypeInformation (TypeInformation):
         id = apply( m, args, kw ) or id  # allow factory to munge ID
         ob = container._getOb( id )
 
-        if hasattr(ob, '_setPortalTypeName'):
-            ob._setPortalTypeName(self.getId())
-        wf = getToolByName(ob, 'portal_workflow', None)
-        if wf is not None:
-            wf.notifyCreated(ob)
-        return ob
+        return self._finishConstruction(ob)
 
 InitializeClass( FactoryTypeInformation )
 
@@ -474,8 +553,7 @@ class ScriptableTypeInformation( TypeInformation ):
     def constructInstance( self, container, id, *args, **kw ):
         """
         Build a "bare" instance of the appropriate type in
-        'container', using 'id' as its id.  Return the URL
-        of its "immediate" view (typically the metadata form).
+        'container', using 'id' as its id.  Return the object.
         """
         if not self.isConstructionAllowed(container):
             raise Unauthorized
@@ -486,10 +564,8 @@ class ScriptableTypeInformation( TypeInformation ):
 
         id = str(id)
         ob = apply(constructor, (container, id) + args, kw)
-        if hasattr(ob, '_setPortalTypeName'):
-            ob._setPortalTypeName(self.getId())
 
-        return ob
+        return self._finishConstruction(ob)
 
 InitializeClass( ScriptableTypeInformation )
 
@@ -499,26 +575,41 @@ ContentFactoryMetadata = FactoryTypeInformation
 ContentTypeInformation = ScriptableTypeInformation
 
 
-allowedTypes = ( 'Script (Python)'
-               , 'Python Method'
-               , 'DTML Method'
-               , 'External Method'
-               , FactoryTypeInformation.meta_type
-               , ScriptableTypeInformation.meta_type
-               )
+typeClasses = [
+    {'class':FactoryTypeInformation,
+     'name':FactoryTypeInformation.meta_type,
+     'action':'manage_addFactoryTIForm',
+     'permission':'Manage portal'},
+    {'class':ScriptableTypeInformation,
+     'name':ScriptableTypeInformation.meta_type,
+     'action':'manage_addScriptableTIForm',
+     'permission':'Manage portal'},
+    ]
 
-class TypesTool( UniqueObject, OFS.Folder.Folder ):
+
+allowedTypes = [
+    'Script (Python)',
+    'Python Method',
+    'DTML Method',
+    'External Method',
+    ]
+
+
+class TypesTool( UniqueObject, OFS.Folder.Folder, ActionProviderBase ):
     """
         Provides a configurable registry of portal content types.
     """
     id = 'portal_types'
     meta_type = 'CMF Types Tool'
+    _actions = []
 
     security = ClassSecurityInfo()
 
-    manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' }
+    manage_options = ( OFS.Folder.Folder.manage_options +
+                      ActionProviderBase.manage_options +
+                      ({ 'label' : 'Overview', 'action' : 'manage_overview' }
                      , 
-                     ) + OFS.Folder.Folder.manage_options
+                     ))
 
     #
     #   ZMI methods
@@ -527,23 +618,31 @@ class TypesTool( UniqueObject, OFS.Folder.Folder ):
                              , 'manage_overview' )
     manage_overview = DTMLFile( 'explainTypesTool', _dtmldir )
 
+    security.declarePrivate('listActions')
+    def listActions(self, info=None):
+        """
+        Return a list of action information instances
+        for actions provided via tool
+        """
+        return self._actions
+
     def all_meta_types(self):
+        """Adds TypesTool-specific meta types."""
         all = TypesTool.inheritedAttribute('all_meta_types')(self)
-        return (
-            {'name':FactoryTypeInformation.meta_type,
-             'action':'manage_addFactoryTIForm',
-             'permission':'Manage portal'},
-            {'name':ScriptableTypeInformation.meta_type,
-             'action':'manage_addScriptableTIForm',
-             'permission':'Manage portal'},
-            ) + tuple(all)
+        return tuple(typeClasses) + tuple(all)
 
     def filtered_meta_types(self, user=None):
         # Filters the list of available meta types.
+        allowed = {}
+        for tc in typeClasses:
+            allowed[tc['name']] = 1
+        for name in allowedTypes:
+            allowed[name] = 1
+
         all = TypesTool.inheritedAttribute('filtered_meta_types')(self)
         meta_types = []
         for meta_type in self.all_meta_types():
-            if meta_type['name'] in allowedTypes:
+            if allowed.get(meta_type['name']):
                 meta_types.append(meta_type)
         return meta_types
 
@@ -574,17 +673,21 @@ class TypesTool( UniqueObject, OFS.Folder.Folder ):
     security.declareProtected(ManagePortal, 'manage_addFactoryTIForm')
     def manage_addFactoryTIForm(self, REQUEST):
         ' '
-        return self._addTIForm(self, REQUEST, scriptable='',
-                               types=self.listDefaultTypeInformation())
+        return self._addTIForm(
+            self, REQUEST,
+            add_meta_type=FactoryTypeInformation.meta_type,
+            types=self.listDefaultTypeInformation())
 
     security.declareProtected(ManagePortal, 'manage_addScriptableTIForm')
     def manage_addScriptableTIForm(self, REQUEST):
         ' '
-        return self._addTIForm(self, REQUEST, scriptable='1',
-                               types=self.listDefaultTypeInformation())
+        return self._addTIForm(
+            self, REQUEST,
+            add_meta_type=ScriptableTypeInformation.meta_type,
+            types=self.listDefaultTypeInformation())
 
     security.declareProtected(ManagePortal, 'manage_addTypeInformation')
-    def manage_addTypeInformation(self, id=None, scriptable='',
+    def manage_addTypeInformation(self, add_meta_type, id=None,
                                   typeinfo_name=None, RESPONSE=None):
         """
         Create a TypeInformation in self.
@@ -602,10 +705,13 @@ class TypesTool( UniqueObject, OFS.Folder.Folder ):
                 id = fti.get('id', None)
         if not id:
             raise 'Bad Request', 'An id is required.'
-        if scriptable:
-            klass = ScriptableTypeInformation
+        for mt in typeClasses:
+            if mt['name'] == add_meta_type:
+                klass = mt['class']
+                break
         else:
-            klass = FactoryTypeInformation
+            raise ValueError, (
+                'Meta type %s is not a type class.' % add_meta_type)
         id = str(id)
         if fti is not None:
             fti = fti.copy()
@@ -625,19 +731,28 @@ class TypesTool( UniqueObject, OFS.Folder.Folder ):
             TypeInformation interface, corresponding to
             the specified 'contentType'.  If contentType is actually
             an object, rather than a string, attempt to look up
-            the appropriate type info using its Type or meta_type.
+            the appropriate type info using its portal_type.
         """
         if type( contentType ) is not type( '' ):
-            try:
+            if hasattr(aq_base(contentType), '_getPortalTypeName'):
                 contentType = contentType._getPortalTypeName()
-            except: # if we can't get or call it for any reason, fall back...
-                contentType = contentType.meta_type
+                if contentType is None:
+                    return None
+            else:
+                return None
         ob = getattr( self, contentType, None )
         if getattr(aq_base(ob), '_isTypeInformation', 0):
             return ob
         else:
             return None
 
+    security.declarePrivate('_checkViewType')
+    def _checkViewType(self,t):
+        try:
+            return getSecurityManager().validate(t, t, 'Title', t.Title)
+        except Unauthorized:
+            return 0
+
     security.declareProtected(AccessContentsInformation, 'listTypeInfo')
     def listTypeInfo( self, container=None ):
         """
@@ -651,9 +766,13 @@ class TypesTool( UniqueObject, OFS.Folder.Folder ):
             # types for which the user does not have adequate permission.
             if not getattr(aq_base(t), '_isTypeInformation', 0):
                 continue
-            if not t.Type():
+            if not t.getId():
+                # XXX What's this used for ?
                 # Not ready.
                 continue
+            # check we're allowed to access the type object
+            if not self._checkViewType(t):
+                continue
             if container is not None:
                 if not t.isConstructionAllowed(container):
                     continue
@@ -663,7 +782,7 @@ class TypesTool( UniqueObject, OFS.Folder.Folder ):
     security.declareProtected(AccessContentsInformation, 'listContentTypes')
     def listContentTypes( self, container=None, by_metatype=0 ):
         """
-            Return list of content metatypes.
+            Return list of content types.
         """
         typenames = {}
         for t in self.listTypeInfo( container ):
@@ -671,16 +790,16 @@ class TypesTool( UniqueObject, OFS.Folder.Folder ):
             if by_metatype:
                 name = t.Metatype()
             else:
-                name = t.Type()
+                name = t.getId()
 
             if name:
-                typenames[ name ] = 1
+                typenames[ name ] = None
 
         result = typenames.keys()
         result.sort()
         return result
 
-    
+
     security.declarePublic('constructContent')
     def constructContent( self
                         , type_name
@@ -697,10 +816,12 @@ class TypesTool( UniqueObject, OFS.Folder.Folder ):
         info = self.getTypeInfo( type_name )
         if info is None:
             raise 'ValueError', 'No such content type: %s' % type_name
-        
-        ob = apply(info.constructInstance, (container, id) + args, kw)
 
-        ob.reindexObject()
+        # check we're allowed to access the type object
+        if not self._checkViewType(info):
+            raise Unauthorized,info
+
+        ob = apply(info.constructInstance, (container, id) + args, kw)
 
         if RESPONSE is not None:
             immediate_url = '%s/%s' % ( ob.absolute_url()
