@@ -1,7 +1,8 @@
 ## Script (Python) "discussion_reply"
-##parameters=title,text,Creator
+##parameters=title,text
 ##title=Reply to content
 
+Creator = context.portal_membership.getAuthenticatedMember().getUserName()
 replyID = context.createReply( title = title
                              , text = text
                              , Creator = Creator
